import { DirectiveLocationEnum, GraphQLNamedType, GraphQLSchema, GraphQLObjectType } from "graphql";
import * as Types from "./types";
export interface DirectiveUse {
    location: DirectiveLocationEnum;
    typeName: string;
    args: [];
    argName?: string;
    fieldName?: string;
}
/**
 * Passed into the SchemaBuilder, this keeps track of any necessary
 * field / type metadata we need to be aware of when building the
 * generated types and/or SDL artifact, including but not limited to:
 *
 * - Directive usage
 * - The TS type backing the GraphQL type
 * - Type default resolver
 * - Whether the field has a resolver
 * - The property name of the field
 * - If the field has a "default" value
 */
export declare class Metadata {
    protected config: Types.BuilderConfig;
    protected completed: boolean;
    protected objectMeta: Record<string, Types.ObjectTypeConfig>;
    protected interfaceMeta: Record<string, Types.InterfaceTypeConfig>;
    protected objectFieldMeta: Record<string, Record<string, Types.OutputFieldConfig>>;
    protected typeImports: string[];
    protected typegenFile: string;
    constructor(config: Types.BuilderConfig);
    finishConstruction(): void;
    /**
     * Ensure the type doesn't conflict with an existing type, prefixing
     * with a _ if it does
     */
    safeTypeName(schema: GraphQLSchema, typeName: string): string;
    /**
     * Check for the field's existence in an object type.
     */
    hasField(schema: GraphQLSchema, typeName: string, fieldName: string): boolean;
    isExternalType(typeName: string): boolean;
    hasResolver(typeName: string, fieldName: string): boolean;
    hasDefaultResolver(typeName: string): boolean;
    hasDefaultValue(type: GraphQLObjectType, fieldName: string): boolean;
    isFieldModified(typeName: string, fieldName: string): boolean;
    isInterfaceField(type: GraphQLObjectType, fieldName: string): boolean;
    addScalar(config: Types.ScalarOpts): void;
    addInterfaceType(config: Types.InterfaceTypeConfig): void;
    addObjectType(config: Types.ObjectTypeConfig): void;
    addExternalType(type: GraphQLNamedType): void;
    addField(typeName: string, field: Types.FieldConfig): void;
    /**
     * Generates the artifacts of the build based on what we
     * know about the schema and how it was defined.
     */
    generateArtifacts(schema: GraphQLSchema): void;
    sortSchema(schema: GraphQLSchema): GraphQLSchema;
    writeFile(type: "schema" | "types", output: string, filePath: string): Promise<void>;
    /**
     * Generates the schema, adding any directives as necessary
     */
    generateSchemaFile(schema: GraphQLSchema): string;
    /**
     * Generates the type definitions
     */
    generateTypesFile(schema: GraphQLSchema): Promise<string>;
    getTypegenInfo(schema: GraphQLSchema): Promise<Types.TypegenInfo>;
    protected checkMutable(): void;
}
