"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var utils_1 = require("./utils");
function convertSDL(sdl, commonjs) {
    if (commonjs === void 0) { commonjs = false; }
    return new SDLConverter(commonjs).convert(sdl);
}
exports.convertSDL = convertSDL;
/**
 * Convert an existing SDL schema into a GraphQL Nexus format
 */
var SDLConverter = /** @class */ (function () {
    function SDLConverter(commonjs) {
        if (commonjs === void 0) { commonjs = false; }
        this.export = commonjs ? "exports." : "export const ";
    }
    SDLConverter.prototype.convert = function (sdl) {
        var _this = this;
        var schema = graphql_1.buildSchema(sdl);
        if (typeof graphql_1.lexicographicSortSchema === "function") {
            schema = graphql_1.lexicographicSortSchema(schema);
        }
        var typeMap = schema.getTypeMap();
        var typeFragments = [];
        Object.keys(typeMap).forEach(function (typeName) {
            if (typeName.indexOf("__") === 0) {
                return;
            }
            var type = typeMap[typeName];
            if (graphql_1.isObjectType(type)) {
                typeFragments.push(_this.makeObjectType(type));
            }
            else if (graphql_1.isInterfaceType(type)) {
                typeFragments.push(_this.makeInterfaceType(type));
            }
            else if (graphql_1.isUnionType(type)) {
                typeFragments.push(_this.makeUnionType(type));
            }
            else if (graphql_1.isEnumType(type)) {
                typeFragments.push(_this.makeEnumType(type));
            }
            else if (graphql_1.isScalarType(type)) {
                typeFragments.push(_this.makeScalarType(type));
            }
            else if (graphql_1.isInputObjectType(type)) {
                typeFragments.push(_this.makeInputObjectType(type));
            }
        });
        return typeFragments.join("\n\n");
    };
    SDLConverter.prototype.makeObjectType = function (type) {
        var str = [
            "" + this.export + type.name + " = objectType(\"" + type.name + "\", t => {",
        ];
        if (type.getInterfaces().length > 0) {
            str.push("  t.implements(\"" + type
                .getInterfaces()
                .map(function (i) { return i.name; })
                .join('", "') + "\")");
        }
        Object.keys(type.getFields()).forEach(function (fieldName) {
            if (utils_1.isInterfaceField(type, fieldName)) {
                return;
            }
            utils_1.eachObj(type.getFields(), function (field, key) {
                getFieldType(field);
            });
        });
        return str.concat("});").join("\n");
    };
    SDLConverter.prototype.makeEnumType = function (type) {
        var str = [
            "" + this.export + type.name + " = enumType(\"" + type.name + "\", t => {",
        ];
        return str.concat("});").join("\n");
    };
    SDLConverter.prototype.makeInterfaceType = function (type) {
        var str = [
            "" + this.export + type.name + " = interfaceType(\"" + type.name + "\", t => {",
        ];
        utils_1.eachObj(type.getFields(), function (field, key) {
            getFieldType(field);
        });
        return str.concat("});").join("\n");
    };
    SDLConverter.prototype.makeInputObjectType = function (type) {
        var str = [
            "" + this.export + type.name + " = inputObjectType(\"" + type.name + "\", t => {",
        ];
        return str.concat("});").join("\n");
    };
    SDLConverter.prototype.makeUnionType = function (type) {
        var str = [
            "" + this.export + type.name + " = unionType(\"" + type.name + "\", t => {",
        ];
        return str.concat("});").join("\n");
    };
    SDLConverter.prototype.makeScalarType = function (type) {
        var str = [
            "" + this.export + type.name + " = scalarType(\"" + type.name + "\", t => {",
        ];
        return str.concat("});").join("\n");
    };
    return SDLConverter;
}());
exports.SDLConverter = SDLConverter;
var getFieldType = function (type) {
    //
};
var getInputFieldType = function (type) {
    //
};
//# sourceMappingURL=sdlConverter.js.map