"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var graphql_1 = require("graphql");
var utils_1 = require("./utils");
// This is intentionally concise and procedural. Didn't want to make this
// unnecessarily abstracted and it should run only with only a single iteration
// of the schema. The types created here are generally meant for internal use
// by GraphQLNexus anyway. If there is a need, this could be factored out into
// something nicer, but for now let's just make it work well, however ugly it is.
// At least it's type safe™
function buildTypeDefinitions(schema, metadata) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, headers, backingTypeMap, contextType, imports, schemaTypeMap, typeNames, allTypeStrings, interfaceRootTypes, returnTypeFields, argTypeFields, MP, MPL, MT, MTA, maybePromiseList, maybePromise, maybeThunk, maybeThunkWithArgs, suffixed, argFieldName, fieldReturnTypeName, typeReturnTypeName, typeRootTypeName, fieldResolverName, getReturnType, printInputType, printArgOrFieldMember, makeFieldArgs, processField, fieldRootType, fieldBackingName, makeRootType, makeReturnType, makeResolvers, objectNames, interfacesWithMembers, enums, unions, scalars, inputObjects;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, metadata.getTypegenInfo(schema)];
                case 1:
                    _a = _b.sent(), headers = _a.headers, backingTypeMap = _a.backingTypeMap, contextType = _a.contextType, imports = _a.imports;
                    schemaTypeMap = schema.getTypeMap();
                    typeNames = {
                        enums: [],
                        objects: [],
                        inputObjects: [],
                        interfaces: [],
                        scalars: [],
                        unions: [],
                    };
                    allTypeStrings = [];
                    interfaceRootTypes = {};
                    returnTypeFields = {};
                    argTypeFields = {};
                    MP = metadata.safeTypeName(schema, "MaybePromise");
                    MPL = metadata.safeTypeName(schema, "MaybePromiseList");
                    MT = metadata.safeTypeName(schema, "MaybeThunk");
                    MTA = metadata.safeTypeName(schema, "MaybeThunkArgs");
                    maybePromiseList = function (t) { return MPL + "<" + t + ">"; };
                    maybePromise = function (t) { return MP + "<" + t + ">"; };
                    maybeThunk = function (t) { return MT + "<" + t + ">"; };
                    maybeThunkWithArgs = function (t, a) { return MTA + "<" + t + ", " + a + ">"; };
                    suffixed = function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        var nameStr = "";
                        if (args.length === 3) {
                            var typeName = args[0], fieldName = args[1], suffix = args[2];
                            var pascalField = pascalCase(fieldName);
                            if (metadata.hasField(schema, typeName, pascalField)) {
                                nameStr = "" + typeName + fieldName + suffix;
                            }
                            else {
                                nameStr = "" + typeName + pascalField + suffix;
                            }
                        }
                        else {
                            nameStr = "" + args[0] + args[1];
                        }
                        return metadata.safeTypeName(schema, nameStr);
                    };
                    argFieldName = function (typeName, fieldName) {
                        return suffixed(typeName, fieldName, "Args");
                    };
                    fieldReturnTypeName = function (typeName, fieldName) {
                        return suffixed(typeName, fieldName, "ReturnType");
                    };
                    typeReturnTypeName = function (type) {
                        if (graphql_1.isUnionType(type)) {
                            return suffixed(type.name, "ReturnType");
                        }
                        if (!utils_1.hasField(type, "ReturnType")) {
                            return suffixed(type.name, "_ReturnType");
                        }
                        return suffixed(type.name, "ReturnType");
                    };
                    typeRootTypeName = function (typeName) {
                        if (isEntryType(typeName)) {
                            return "{}";
                        }
                        return suffixed(typeName, "RootType");
                    };
                    fieldResolverName = function (typeName, fieldName) {
                        return suffixed(typeName, fieldName, "Resolver");
                    };
                    getReturnType = function (fieldType) {
                        var _a = unwrapNull(fieldType), type = _a.type, typeStr = _a.typeStr;
                        if (graphql_1.isListType(type)) {
                            return "" + typeStr + maybePromiseList(getReturnType(type.ofType));
                        }
                        if (graphql_1.isObjectType(type) || graphql_1.isInterfaceType(type) || graphql_1.isUnionType(type)) {
                            typeStr += typeReturnTypeName(type);
                        }
                        else if (graphql_1.isEnumType(type)) {
                            typeStr += type.name;
                        }
                        else if (graphql_1.isScalarType(type)) {
                            typeStr += backingTypeMap[type.name] || "unknown";
                        }
                        return typeStr;
                    };
                    printInputType = function (fieldType) {
                        var _a = unwrapNull(fieldType), type = _a.type, typeStr = _a.typeStr;
                        if (graphql_1.isListType(type)) {
                            var inputTypeStr = printInputType(type.ofType);
                            return inputTypeStr.indexOf("null ") !== -1
                                ? typeStr + "Array<" + inputTypeStr + ">"
                                : "" + typeStr + inputTypeStr + "[]";
                        }
                        if (graphql_1.isInputObjectType(type) || graphql_1.isEnumType(type)) {
                            return "" + typeStr + type.name;
                        }
                        if (graphql_1.isScalarType(type)) {
                            return "" + typeStr + (backingTypeMap[type.name] || "unknown");
                        }
                        throw new Error("Unexpected type " + type);
                    };
                    printArgOrFieldMember = function (_a) {
                        var name = _a.name, type = _a.type;
                        if (graphql_1.isNonNullType(type)) {
                            return "  " + name + ": " + printInputType(type) + ";";
                        }
                        return "  " + name + "?: " + printInputType(type) + ";";
                    };
                    makeFieldArgs = function (argTypeName, field) {
                        allTypeStrings.push([
                            "export interface " + argTypeName + " {",
                            map(field.args, function (arg) { return printArgOrFieldMember(arg); }),
                            "}",
                        ].join("\n"));
                    };
                    processField = function (type, field) {
                        var returnTypeName = fieldReturnTypeName(type.name, field.name);
                        utils_1.arrPush(returnTypeFields, type.name, [field.name, returnTypeName]);
                        allTypeStrings.push("export type " + returnTypeName + " = " + getReturnType(field.type) + ";");
                        if (field.args.length) {
                            var argTypeName = argFieldName(type.name, field.name);
                            utils_1.arrPush(argTypeFields, type.name, [field.name, argTypeName]);
                            if (graphql_1.isObjectType(type)) {
                                var interfaces = type
                                    .getInterfaces()
                                    .filter(function (i) { return i.getFields()[field.name]; })
                                    .map(function (i) { return argFieldName(i.name, field.name); });
                                if (interfaces.length) {
                                    allTypeStrings.push("export interface " + argTypeName + " extends " + interfaces.join(", ") + " {}");
                                }
                                else {
                                    makeFieldArgs(argTypeName, field);
                                }
                            }
                            else {
                                makeFieldArgs(argTypeName, field);
                            }
                        }
                    };
                    fieldRootType = function (fieldType) {
                        var _a = unwrapNull(fieldType), type = _a.type, typeStr = _a.typeStr;
                        if (graphql_1.isListType(type)) {
                            var toWrap = fieldRootType(type.ofType);
                            return toWrap.indexOf("null | ") === 0
                                ? typeStr + "Array<" + toWrap + ">"
                                : "" + typeStr + toWrap + "[]";
                        }
                        if (graphql_1.isScalarType(type)) {
                            return "" + typeStr + (backingTypeMap[type.name] || "any");
                        }
                        if (graphql_1.isObjectType(type)) {
                            // return field.
                        }
                        if (graphql_1.isEnumType(type)) {
                            return "" + typeStr + type.name;
                        }
                        return typeStr + "any";
                    };
                    fieldBackingName = function (type, field) {
                        var colon = metadata.hasDefaultValue(type, field.name)
                            ? "?:"
                            : graphql_1.isNonNullType(field.type)
                                ? ":"
                                : "?:";
                        return "" + field.name + colon;
                    };
                    makeRootType = function (type) {
                        if (isEntryType(type.name)) {
                            return;
                        }
                        if (backingTypeMap[type.name]) {
                            allTypeStrings.push("export type " + typeRootTypeName(type.name) + " = " + backingTypeMap[type.name] + ";");
                        }
                        else {
                            var rootMembers = utils_1.mapObj(type.getFields(), function (f) {
                                if (metadata.hasResolver(type.name, f.name)) {
                                    return null;
                                }
                                return "  " + fieldBackingName(type, f) + " " + fieldRootType(f.type) + ";";
                            }).filter(function (f) { return f; });
                            if (rootMembers.length === 0) {
                                allTypeStrings.push("export type " + typeRootTypeName(type.name) + " = {};");
                            }
                            else {
                                allTypeStrings.push([
                                    "export interface " + typeRootTypeName(type.name) + " {",
                                    rootMembers.join("\n"),
                                    "}",
                                ].join("\n"));
                            }
                        }
                    };
                    makeReturnType = function (type) {
                        if (backingTypeMap[type.name]) {
                            allTypeStrings.push("export type " + typeReturnTypeName(type) + " = " + backingTypeMap[type.name]);
                        }
                        else {
                            var returnMembers = utils_1.mapObj(type.getFields(), function (f) {
                                var hasArgs = f.args.length > 0;
                                if (metadata.hasResolver(type.name, f.name)) {
                                    return null;
                                }
                                var rootType = fieldRootType(f.type);
                                return "  " + fieldBackingName(type, f) + " " + (hasArgs
                                    ? maybeThunkWithArgs(maybePromise(rootType), argFieldName(type.name, f.name))
                                    : maybeThunk(maybePromise(rootType))) + ";";
                            }).filter(function (f) { return f; });
                            if (returnMembers.length === 0) {
                                allTypeStrings.push("export type " + typeReturnTypeName(type) + " = {};");
                            }
                            else {
                                allTypeStrings.push([
                                    "export type " + typeReturnTypeName(type) + " = {",
                                    returnMembers.join("\n"),
                                    "}",
                                ].join("\n"));
                            }
                        }
                    };
                    makeResolvers = function (type) {
                        // TODO
                    };
                    Object.keys(schemaTypeMap).forEach(function (typeName) {
                        if (typeName.indexOf("__") === 0) {
                            return;
                        }
                        // All types will be "unused" until we say otherwise,
                        // if we need to strip them.
                        var type = schema.getType(typeName);
                        // An object type has a "backing type", and fields
                        // which may have an "arg type" and have a "return type"
                        if (graphql_1.isObjectType(type)) {
                            typeNames.objects.push(type.name);
                            utils_1.eachObj(type.getFields(), function (field) { return processField(type, field); });
                            type
                                .getInterfaces()
                                .forEach(function (i) { return utils_1.arrPush(interfaceRootTypes, i.name, type); });
                            makeRootType(type);
                            makeReturnType(type);
                            makeResolvers(type);
                        }
                        else if (graphql_1.isInputObjectType(type)) {
                            typeNames.inputObjects.push(type.name);
                            allTypeStrings.push([
                                "export interface " + type.name + " {",
                                utils_1.mapObj(type.getFields(), function (inputField) {
                                    return printArgOrFieldMember(inputField);
                                }).join("\n"),
                                "}",
                            ].join("\n"));
                        }
                        else if (graphql_1.isScalarType(type)) {
                            typeNames.scalars.push(type.name);
                        }
                        else if (graphql_1.isUnionType(type)) {
                            typeNames.unions.push(type.name);
                            allTypeStrings.push("export type " + typeRootTypeName(type.name) + " = " + map(type.getTypes(), function (_a) {
                                var name = _a.name;
                                return typeRootTypeName(name);
                            }, " | ") + ";");
                            allTypeStrings.push("export type " + typeReturnTypeName(type) + " = " + map(type.getTypes(), function (t) { return typeReturnTypeName(t); }, " | ") + ";");
                        }
                        else if (graphql_1.isInterfaceType(type)) {
                            typeNames.interfaces.push(type.name);
                            utils_1.eachObj(type.getFields(), function (field) { return processField(type, field); });
                        }
                        else if (graphql_1.isEnumType(type)) {
                            typeNames.enums.push(type.name);
                            allTypeStrings.push("export type " + type.name + " = " + map(type.getValues(), function (_a) {
                                var value = _a.value;
                                return JSON.stringify(value);
                            }, " | ") + ";");
                        }
                    });
                    utils_1.eachObj(interfaceRootTypes, function (members, interfaceName) {
                        var i = schema.getType(interfaceName);
                        if (!graphql_1.isInterfaceType(i)) {
                            return;
                        }
                        if (backingTypeMap[interfaceName]) {
                            allTypeStrings.push("export type " + typeRootTypeName(interfaceName) + " = " + backingTypeMap[interfaceName] + ";");
                            allTypeStrings.push("export type " + typeReturnTypeName(i) + " = " + backingTypeMap[interfaceName] + ";");
                        }
                        else {
                            allTypeStrings.push("export type " + typeRootTypeName(interfaceName) + " = " + members
                                .map(function (_a) {
                                var name = _a.name;
                                return typeRootTypeName(name);
                            })
                                .join(" | ") + ";");
                            allTypeStrings.push("export type " + typeReturnTypeName(i) + " = " + members
                                .map(function (name) { return typeReturnTypeName(name); })
                                .join(" | ") + ";");
                        }
                    });
                    objectNames = function () {
                        return [
                            "{",
                            map(typeNames.objects, function (n) { return "    " + n + ": " + typeRootTypeName(n) + ";"; }),
                            "  }",
                        ].join("\n");
                    };
                    interfacesWithMembers = function () {
                        if (typeNames.interfaces.length === 0) {
                            return "{}";
                        }
                        return [
                            "{",
                            map(typeNames.interfaces, function (n) {
                                return "    " + n + ": " + (interfaceRootTypes[n]
                                    ? interfaceRootTypes[n].map(stringify).join(" | ")
                                    : "never") + ";";
                            }),
                            "  }",
                        ].join("\n");
                    };
                    enums = function () {
                        if (typeNames.enums.length === 0) {
                            return "{}";
                        }
                        return ["{", map(typeNames.enums, function (n) { return "    " + n + ": " + n + ";"; }), "  }"].join("\n");
                    };
                    unions = function () {
                        if (typeNames.unions.length === 0) {
                            return "{}";
                        }
                        return ["{", map(typeNames.unions, function (n) { return "    " + n + ": any;"; }), "  }"].join("\n");
                    };
                    scalars = function () {
                        return ["{", map(typeNames.scalars, function (n) { return "    " + n + ": any;"; }), "  }"].join("\n");
                    };
                    inputObjects = function () {
                        if (typeNames.inputObjects.length === 0) {
                            return "{}";
                        }
                        return [
                            "{",
                            map(typeNames.inputObjects, function (n) { return "    " + n + ": " + n + ";"; }),
                            "  }",
                        ].join("\n");
                    };
                    return [2 /*return*/, headers.join("\n") + "\n" + imports.join("\n") + "\n\ndeclare global {\n  interface GraphQLNexusGen extends GraphQLNexusGenTypes {}\n}\n\n// Maybe Promise\nexport type " + MP + "<T> = T | PromiseLike<T>;\n\n// Maybe Promise List\nexport type " + MPL + "<T> = Array<" + MP + "<T>>;\n\n// Maybe Thunk\nexport type " + MT + "<T> = T | (() => T);\n\n// Maybe Thunk, with args\nexport type " + MTA + "<T, A> = T | ((args?: A) => T);\n\n" + allTypeStrings.join("\n\n") + "\n\n" + stringifyTypeFieldMapping("GraphQLNexusGenArgTypes", argTypeFields) + "\n\nexport interface GraphQLNexusGenRootTypes {\n" + map(typeNames.interfaces.concat(typeNames.objects), function (name) { return "  " + name + ": " + typeRootTypeName(name) + ";"; }) + "\n}\n\n" + stringifyTypeFieldMapping("GraphQLNexusGenReturnTypes", returnTypeFields) + "\n\nexport interface GraphQLNexusGenTypes {\n  argTypes: GraphQLNexusGenArgTypes;\n  backingTypes: GraphQLNexusGenRootTypes;\n  returnTypes: GraphQLNexusGenReturnTypes;\n  context: " + contextType + ";\n  enums: " + enums() + ";\n  objects: " + objectNames() + ";\n  interfaces: " + interfacesWithMembers() + ";\n  unions: " + unions() + ";\n  scalars: " + scalars() + ";\n  inputObjects: " + inputObjects() + ";\n  allInputTypes: \n    | Extract<keyof GraphQLNexusGenTypes['inputObjects'], string>\n    | Extract<keyof GraphQLNexusGenTypes['enums'], string>\n    | Extract<keyof GraphQLNexusGenTypes['scalars'], string>;\n  allOutputTypes: \n    | Extract<keyof GraphQLNexusGenTypes['objects'], string>\n    | Extract<keyof GraphQLNexusGenTypes['enums'], string>\n    | Extract<keyof GraphQLNexusGenTypes['unions'], string>\n    | Extract<keyof GraphQLNexusGenTypes['interfaces'], string>\n    | Extract<keyof GraphQLNexusGenTypes['scalars'], string>;\n}\n\nexport type Gen = GraphQLNexusGenTypes;\n"];
            }
        });
    });
}
exports.buildTypeDefinitions = buildTypeDefinitions;
function isEntryType(typeName) {
    return (typeName === "Query" ||
        typeName === "Mutation" ||
        typeName === "Subscription");
}
var stringify = function (v) { return JSON.stringify(v); };
function stringifyTypeFieldMapping(tsInterfaceName, obj) {
    var argTypeLines = Object.keys(obj).reduce(function (result, typeName) {
        return result
            .concat("  " + typeName + ": {")
            .concat(obj[typeName].reduce(function (fields, _a) {
            var fieldName = _a[0], mappingName = _a[1];
            return fields.concat("    " + fieldName + ": " + mappingName + ";");
        }, []))
            .concat("  };");
    }, []);
    var argTypes = ["export interface " + tsInterfaceName + " {"]
        .concat(argTypeLines)
        .concat("}")
        .join("\n");
    return argTypes;
}
function map(nodes, iterator, join) {
    if (join === void 0) { join = "\n"; }
    return Array.from(nodes)
        .map(iterator)
        .join(join);
}
function ucFirst(str) {
    return str
        .slice(0, 1)
        .toUpperCase()
        .concat(str.slice(1));
}
function camelize(str) {
    return str.replace(/^([A-Z])|[\s-_]+(\w)/g, function (match, p1, p2, offset) {
        if (p2) {
            return p2.toUpperCase();
        }
        return p1.toLowerCase();
    });
}
function pascalCase(str) {
    return ucFirst(camelize(str));
}
function unwrapNull(fieldType) {
    var type = fieldType;
    var typeStr = "";
    if (graphql_1.isNonNullType(fieldType)) {
        type = fieldType.ofType;
    }
    else {
        typeStr += "null | ";
    }
    return { type: type, typeStr: typeStr };
}
//# sourceMappingURL=typegen.js.map