"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
function log(msg) {
    console.log("GraphQL Nexus: " + msg);
}
exports.log = log;
function withDeprecationComment(description) {
    return description;
}
exports.withDeprecationComment = withDeprecationComment;
exports.enumShorthandMembers = function (arg) {
    if (Array.isArray(arg)) {
        return arg.map(function (name) { return ({ name: name, value: name }); });
    }
    return Object.keys(arg).map(function (name) {
        return {
            name: name,
            value: arg[name],
        };
    });
};
exports.isInterfaceField = function (type, fieldName) {
    return type.getInterfaces().some(function (i) { return Boolean(i.getFields()[fieldName]); });
};
exports.hasField = function (type, fieldName) {
    return Boolean(type.getFields()[fieldName]);
};
// ----------------------------
/**
 *
 * Copied from graphql-js:
 *
 */
/**
 * Given an invalid input string and a list of valid options, returns a filtered
 * list of valid options sorted based on their similarity with the input.
 */
function suggestionList(input, options) {
    var optionsByDistance = Object.create(null);
    var oLength = options.length;
    var inputThreshold = input.length / 2;
    for (var i = 0; i < oLength; i++) {
        var distance = lexicalDistance(input, options[i]);
        var threshold = Math.max(inputThreshold, options[i].length / 2, 1);
        if (distance <= threshold) {
            optionsByDistance[options[i]] = distance;
        }
    }
    return Object.keys(optionsByDistance).sort(function (a, b) {
        return optionsByDistance[a] - optionsByDistance[b];
    });
}
exports.suggestionList = suggestionList;
/**
 * Computes the lexical distance between strings A and B.
 *
 * The "distance" between two strings is given by counting the minimum number
 * of edits needed to transform string A into string B. An edit can be an
 * insertion, deletion, or substitution of a single character, or a swap of two
 * adjacent characters.
 *
 * Includes a custom alteration from Damerau-Levenshtein to treat case changes
 * as a single edit which helps identify mis-cased values with an edit distance
 * of 1.
 *
 * This distance can be useful for detecting typos in input or sorting
 */
function lexicalDistance(aStr, bStr) {
    if (aStr === bStr) {
        return 0;
    }
    var i;
    var j;
    var d = [];
    var a = aStr.toLowerCase();
    var b = bStr.toLowerCase();
    var aLength = a.length;
    var bLength = b.length; // Any case change counts as a single edit
    if (a === b) {
        return 1;
    }
    for (i = 0; i <= aLength; i++) {
        d[i] = [i];
    }
    for (j = 1; j <= bLength; j++) {
        d[0][j] = j;
    }
    for (i = 1; i <= aLength; i++) {
        for (j = 1; j <= bLength; j++) {
            var cost = a[i - 1] === b[j - 1] ? 0 : 1;
            d[i][j] = Math.min(d[i - 1][j] + 1, d[i][j - 1] + 1, d[i - 1][j - 1] + cost);
            if (i > 1 && j > 1 && a[i - 1] === b[j - 2] && a[i - 2] === b[j - 1]) {
                d[i][j] = Math.min(d[i][j], d[i - 2][j - 2] + cost);
            }
        }
    }
    return d[aLength][bLength];
}
// ----------------------------
// Borrowed from https://github.com/dmnd/dedent
function dedent(strings) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    var raw = typeof strings === "string" ? [strings] : strings.raw;
    // first, perform interpolation
    var result = "";
    for (var i = 0; i < raw.length; i++) {
        result += raw[i]
            // join lines when there is a suppressed newline
            .replace(/\\\n[ \t]*/g, "")
            // handle escaped backticks
            .replace(/\\`/g, "`");
        if (i < values.length) {
            result += values[i];
        }
    }
    // now strip indentation
    var lines = result.split("\n");
    var mindent = null;
    lines.forEach(function (l) {
        var m = l.match(/^(\s+)\S+/);
        if (m) {
            var indent = m[1].length;
            if (!mindent) {
                // this is the first indented line
                mindent = indent;
            }
            else {
                mindent = Math.min(mindent, indent);
            }
        }
    });
    if (mindent !== null) {
        var m_1 = mindent; // appease Flow
        result = lines.map(function (l) { return (l[0] === " " ? l.slice(m_1) : l); }).join("\n");
    }
    return (result
        // dedent eats leading and trailing whitespace too
        .trim()
        // handle escaped newlines at the end to ensure they don't get stripped too
        .replace(/\\n/g, "\n"));
}
exports.dedent = dedent;
// ----------------------------
// Helper Fns
function arrPush(obj, property, value) {
    obj[property] = obj[property] || [];
    obj[property].push(value);
}
exports.arrPush = arrPush;
function objValues(obj) {
    return Object.keys(obj).reduce(function (result, key) {
        result.push(obj[key]);
        return result;
    }, []);
}
exports.objValues = objValues;
function mapObj(obj, mapper) {
    return Object.keys(obj).map(function (key, index) { return mapper(obj[key], key, index); });
}
exports.mapObj = mapObj;
function eachObj(obj, iter) {
    Object.keys(obj).forEach(function (name, i) { return iter(obj[name], name, i); });
}
exports.eachObj = eachObj;
exports.isObject = function (obj) {
    return obj !== null && typeof obj === "object";
};
exports.assertAbsolutePath = function (pathName, property) {
    if (!path_1.default.isAbsolute(pathName)) {
        throw new Error("Expected path for " + property + " to be a string, saw " + pathName);
    }
    return pathName;
};
//# sourceMappingURL=utils.js.map